-- This project is based on an example from Stuart Padley's Weblog 
-- subtitled "SQL Server 2005 stuff I couldn't find anywhere else"
-- http://blogs.msdn.com/stuartpa/archive/2005/07/25/443229.aspx

-- The following changes have been made to Stuart's code:
-- 1. Split out EXEC SendMessageProc from Stage 4 and all of Stage 5 into a separate batch 
--    (SendMessageToTarget.sql) to enable sending multiple messages without recreating all objects.
-- 2. Changed BEGIN DIALOG to BEGIN DIALOG CONVERSATION in the SendMessageProc procedure
-- 3. Added a missing END CONVERSATION in the SendMessageProc procedure
-- 4. Several object names have 01 appended to prevent potential conflicts with other
--    sample applications (TargetUser01, TargetService01, InitiatorUser01, InitiatorService0101).

-- YOU MUST MANUALLY DELETE THE CERTIFICATE FILES C:\ServiceBroker\InitiatorCert01.cer
-- and C:\ServiceBroker\TargetCert01.cer BEFORE RUNNING THIS SCRIPT A SECOND TIME

-- STAGE 1: SETUP TARGET DATABASE
USE master;

/* Create Target Database */
IF EXISTS (SELECT * FROM sys.databases WHERE name = 'Target') DROP DATABASE Target;
GO
CREATE DATABASE Target
GO

-- Activate the Broker in this database (can only be done with ALTER DATABASE)
ALTER DATABASE Target SET ENABLE_BROKER;
GO

USE Target; 
GO

-- Create the database master key
CREATE MASTER KEY ENCRYPTION BY PASSWORD = '<Specify a password here>';
GO

/* Set up Target User, and place certificate (without private key) in a file */
-- Create Target User without login
IF EXISTS (SELECT * FROM sys.sysusers WHERE name = 'TargetUser') DROP USER TargetUser;
CREATE USER TargetUser WITHOUT LOGIN;
GO

-- Create a Cert for the Initiator user
IF EXISTS (SELECT * FROM sys.certificates WHERE name = 'TargetCert01') DROP CERTIFICATE TargetCert01;
CREATE CERTIFICATE TargetCert01 
   AUTHORIZATION TargetUser
   WITH SUBJECT = 'Target Certificate', EXPIRY_DATE = '10/31/2009';
GO

-- Backup the cert up so the Initiator database can import the cert (public key)
BACKUP CERTIFICATE TargetCert01 TO FILE = 'C:\ServiceBroker\TargetCert01.cer';
GO

/* Set up SSB */
-- First, we need to create a message type. This message type is very simple and allows any type of content
-- (Drop contract because it binds to message)
IF EXISTS (SELECT * FROM sys.service_contracts WHERE name = 'SubmissionContract') DROP CONTRACT SubmissionContract;
IF EXISTS (SELECT * FROM sys.service_message_types WHERE name = 'Message') DROP MESSAGE TYPE Message;
CREATE MESSAGE TYPE Message VALIDATION = NONE;
GO

-- Now create a contract that specifies what type types of messages
CREATE CONTRACT SubmissionContract (Message SENT BY INITIATOR);
GO

-- Set up Target receive queue to hold messages
-- (Need to drop service before queue, due to binding)
IF EXISTS (SELECT * FROM sys.services WHERE name = 'TargetService01') DROP SERVICE TargetService01;
IF EXISTS (SELECT * FROM sys.service_queues WHERE name = 'TargetQueue') DROP QUEUE TargetQueue;
CREATE QUEUE TargetQueue;
GO

-- Create the required service and bind to be above created queue
CREATE SERVICE TargetService01
   AUTHORIZATION TargetUser
   ON QUEUE TargetQueue (SubmissionContract);
GO

-- STAGE 2: SETUP INITIATOR DATABASE
USE master;

/* Create Initiator Database */
IF EXISTS (SELECT * FROM sys.databases WHERE name = 'Initiator') DROP DATABASE Initiator;
CREATE DATABASE Initiator;
GO

-- Activate the Broker in this database
ALTER DATABASE Initiator SET ENABLE_BROKER;
GO

USE Initiator;
GO

-- Create the database master key
CREATE MASTER KEY ENCRYPTION BY PASSWORD = '<Specify a password here>';
GO

/* Set up Initiator User */
-- Create Initiator User without Login
IF EXISTS (SELECT * FROM sys.sysusers WHERE name = 'InitiatorUser01') DROP USER InitiatorUser01;
CREATE USER InitiatorUser01 WITHOUT LOGIN;
GO

-- Create a Cert with InitiatorUser AUTHORIZATION
-- This links the Cert to the User so SSB uses the CERT for Dialog security
IF EXISTS (SELECT * FROM sys.certificates WHERE name = 'InitiatorCert01') DROP CERTIFICATE InitiatorCert01;
CREATE CERTIFICATE InitiatorCert01 
   AUTHORIZATION InitiatorUser01
   WITH SUBJECT = 'Initiator Certificate 01', EXPIRY_DATE = '10/31/2009';
GO

-- Backup the cert up so the Target can import the cert (public key)
BACKUP CERTIFICATE InitiatorCert01 TO FILE = 'C:\ServiceBroker\InitiatorCert01.cer';
GO

/* Set up SSB */
-- First, we need to create a message type. Note that our message type is
-- very simple and allows any type of content
-- (Need to drop contract before message, due to binding)
IF EXISTS (SELECT * FROM sys.service_contracts WHERE name = 'SubmissionContract') DROP CONTRACT SubmissionContract;
IF EXISTS (SELECT * FROM sys.service_message_types WHERE name = 'Message') DROP MESSAGE TYPE Message;
CREATE MESSAGE TYPE Message VALIDATION = NONE;
GO

-- Once the message type has been created, we need to create a contract
-- that specifies who can send what types of messages
CREATE CONTRACT SubmissionContract (Message SENT BY INITIATOR);

-- Set up Initiator send queue to hold messages
-- (Need to drop service before queue, due to binding)
IF EXISTS (SELECT * FROM sys.services WHERE name = 'InitiatorService01') DROP SERVICE InitiatorService01;
IF EXISTS (SELECT * FROM sys.service_queues WHERE name = 'InitiatorQueue') DROP QUEUE InitiatorQueue;
CREATE QUEUE InitiatorQueue;
GO

-- Create the Initiator service and bind to be above created Initiator queue
CREATE SERVICE InitiatorService01
   AUTHORIZATION InitiatorUser01 ON QUEUE InitiatorQueue;
GO

-- Create a Local Route for the destination TargetService01
IF EXISTS (SELECT * FROM sys.routes WHERE name = 'TargetRoute') DROP ROUTE TargetRoute;
CREATE ROUTE TargetRoute WITH SERVICE_NAME = 'TargetService01', ADDRESS = 'LOCAL'
GO

/* Set up the Target user using the Target EXPORTED CERT  */
-- Create Target User without login
IF EXISTS (SELECT * FROM sys.sysusers WHERE name = 'TargetUser') DROP USER TargetUser;
CREATE USER TargetUser WITHOUT LOGIN;
GO

-- Create a Cert from the external cert file
CREATE CERTIFICATE TargetCert01 
   AUTHORIZATION TargetUser
   FROM FILE = 'C:\ServiceBroker\TargetCert01.cer'
GO

-- Create a remote service binding (only needs to be done on the initator)
CREATE REMOTE SERVICE BINDING TargetBinding
   TO SERVICE 'TargetService01'
   WITH USER = TargetUser
GO

-- STAGE 3: 
use Target;
GO

/* Set up the Target user using the Target EXPORTED CERT  */
-- Create Initiator User without Login
IF EXISTS (SELECT * FROM sys.sysusers WHERE name = 'InitiatorUser01') DROP USER InitiatorUser01;
CREATE USER InitiatorUser01 WITHOUT LOGIN;
GO

-- Create a Cert with InitiatorUser AUTHORIZATION
-- This links the Cert to the User so SSB uses the CERT for Dialog security
IF EXISTS (SELECT * FROM sys.certificates WHERE name = 'InitiatorCert01') DROP CERTIFICATE InitiatorCert01;
-- Create a Cert from the external cert file
CREATE CERTIFICATE InitiatorCert01 
   AUTHORIZATION InitiatorUser01
   FROM FILE = 'C:\ServiceBroker\InitiatorCert01.cer'
GO

GRANT SEND ON SERVICE::TargetService01 TO InitiatorUser01
GO

-- STAGE 4: Create a proc to send SSB Message from Initiator to Target
USE Initiator;
GO

/* Send Proc To Send SSB Message from Initiator to Target */
IF EXISTS (SELECT * FROM sys.objects WHERE name = 'SendMessageProc') DROP PROCEDURE SendMessageProc;    
GO
CREATE PROCEDURE SendMessageProc AS
BEGIN
   DECLARE @conversationHandle UNIQUEIDENTIFIER
   DECLARE @message NVARCHAR(100)
   BEGIN TRANSACTION;
      -- Below changed from BEGIN DIALOG @conversationHandle
      BEGIN DIALOG CONVERSATION @conversationHandle
         FROM SERVICE InitiatorService01
         TO SERVICE 'TargetService01'
         ON CONTRACT SubmissionContract

      -- Send a message on the conversation
      SET @message = N'Service Broker message from Initiator to Target database';
      SEND ON CONVERSATION @conversationHandle
         MESSAGE TYPE Message (@message)
      -- Below was missing, added
	  END CONVERSATION @conversationHandle;
   COMMIT TRANSACTION
END
GO

USE master;
GO